#========================================================================================
# package      DE_EPAGES::Amazon::API::RemoteAdapterFactory
# state        public
# description  Adapter factory for different Amazon remote adapters.
#========================================================================================
package DE_EPAGES::Amazon::API::RemoteAdapterFactory;

use DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter;
use DE_EPAGES::Amazon::API::Object::RemoteMWS;
use DE_EPAGES::Amazon::API::Object::RemoteMWSProduct;
use DE_EPAGES::Core::API::TestParameter qw ( TestObject );

use strict;

sub NewRemoteMWS {
    my ($PortalConfig) = @_;
    TestObject( 'PortalConfig', $PortalConfig, 'DE_EPAGES::ProductPortal::API::Object::PortalConfig' );
    my $PortalSite = $PortalConfig->get('PortalSite');
    my $hInfo = $PortalSite->get( [qw( Portal MWSURL MWSKeyID MWSSecretKeyID MWSMarketplaceID )] );

    return DE_EPAGES::Amazon::API::Object::RemoteMWS->new(
        'AWSURL'         => $hInfo->{'MWSURL'},
        'AWSAccessKeyId' => $hInfo->{'MWSKeyID'},
        'AWSSecretKeyId' => $hInfo->{'MWSSecretKeyID'},
        'Logging'        => $hInfo->{'Portal'}->get('Logging'),
        'MerchantID'     => $PortalConfig->get('MWSMerchantID'),
        'MWSAuthToken'   => $PortalConfig->get('MWSAuthToken'),
        'MarketplaceID'  => $hInfo->{'MWSMarketplaceID'},
    );
}

sub NewRemoteMerchantToMWSAdapter {
    my ($PortalConfig)   = @_;
    TestObject( 'PortalConfig', $PortalConfig, 'DE_EPAGES::ProductPortal::API::Object::PortalConfig' );
    my $PortalSite = $PortalConfig->get('PortalSite');
    my $hInfo = $PortalSite->get( [qw( Portal MWSURL MWSKeyID MWSSecretKeyID MWSMarketplaceID SPAPI_EndPoint )] );

    return DE_EPAGES::Amazon::API::Object::RemoteMerchantToMWSAdapter->new(
        'AWSURL'         => $hInfo->{'MWSURL'},
        'AWSAccessKeyId' => $hInfo->{'MWSKeyID'},
        'AWSSecretKeyId' => $hInfo->{'MWSSecretKeyID'},
        'Logging'        => $hInfo->{'Portal'}->get('Logging'),
        'MerchantID'     => $PortalConfig->get('MWSMerchantID'),
        'MWSAuthToken'   => $PortalConfig->get('MWSAuthToken'),
        'MarketplaceID'  => $hInfo->{'MWSMarketplaceID'},
        'SPAPI_EndPoint' => $hInfo->{'SPAPI_EndPoint'},
        'PortalConfig'   => $PortalConfig,

    );
}

sub NewRemoteMWSProduct {
    my ( $PortalConfig ) = @_;
    TestObject( 'The Var: PortalConfig', $PortalConfig, 'DE_EPAGES::ProductPortal::API::Object::PortalConfig' );
    my $PortalSite = $PortalConfig->get('PortalSite');
    my $hInfo = $PortalSite->get( [qw( Portal MWSURL MWSKeyID MWSSecretKeyID MWSMarketplaceID PortalURL)] );
    my %data = (
        'AWSURL'         => $hInfo->{'MWSURL'}.'Products/2011-10-01',
        'AWSAccessKeyId' => $hInfo->{'MWSKeyID'},
        'AWSSecretKeyId' => $hInfo->{'MWSSecretKeyID'},
        'Logging'        => $hInfo->{'Portal'}->get('Logging'),
        'SellerID'       => $PortalConfig->get('MWSMerchantID'),
        'MWSAuthToken'   => $PortalConfig->get('MWSAuthToken'),
        'MarketplaceID'  => $hInfo->{'MWSMarketplaceID'},
        'PortalURL'      => $hInfo->{'PortalURL'},
    );
    return DE_EPAGES::Amazon::API::Object::RemoteMWSProduct->new(%data);
}

1;
