#========================================================================================
# package      DE_EPAGES::Amazon::Hooks::Attribute
# state        private
#========================================================================================
package DE_EPAGES::Amazon::Hooks::Attribute;

use strict;
use DE_EPAGES::Object::API::Factory qw (LoadObjectByPath LoadRootObject);
use DE_EPAGES::ThirdPartyConfig::API::Constants qw (ATTRIBUTES_CATEGORY_PRODUCTPORTAL);

#========================================================================================
# function     OnGetThirdPartyConfigAttributes
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnGetThirdPartyConfigAttributes($hParams);
#----------------------------------------------------------------------------------------
# description  API_GetThirdPartyConfigAttributes triggered, register attributes to be
#               configured in PBO
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used keys are
#               <ul>
#                   <li>Attributes</li>
#                   <li>AttributesCategory</li>
#                   <li>LanguageID</li>
#               </ul> | ref.hash.*
#========================================================================================
sub OnGetThirdPartyConfigAttributes {
    my ($hParams) = @_;

    return unless $hParams->{'AttributesCategory'} eq ATTRIBUTES_CATEGORY_PRODUCTPORTAL;

    my $Portal = LoadObjectByPath('/Portals/Amazon');

    my @Attributes = (
        { 'ObjectID' => $Portal->id, 'AttributeAlias' => 'Logging' },
        { 'ObjectID' => $Portal->id, 'AttributeAlias' => 'AWSKeyID' },
        { 'ObjectID' => $Portal->id, 'AttributeAlias' => 'AWSSecretKeyID' },
        { 'ObjectID' => $Portal->id, 'AttributeAlias' => 'AWSPriceUpdateInterval' }
    );

    my $ProtocolAndServer = LoadRootObject()->get('ProtocolAndServerSSL');
    foreach my $PortalSite (@{$Portal->get('PortalSites')}) {
        my $IconImage = $PortalSite->get('FlagGif');
        my $IconImageText = undef;
        if (defined $IconImage) {
            $IconImage = $ProtocolAndServer . $IconImage;
            $IconImageText = $PortalSite->get('Country')->{'Native'} . ' / ' . $PortalSite->get('Language')->{'Native'};
        }
        push(
            @Attributes,
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'SPAPI_OAuthRedirectURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'AdminURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'PortalURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'RegisterURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'AWSURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'SellerCentralOrderSearchURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'SOAPURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
            {
                'ObjectID'       => $PortalSite->id,
                'AttributeAlias' => 'AWSImagesHTTPSURL',
                'IconImage'      => $IconImage,
                'IconImageText'  => $IconImageText
            },
        );
    }

    push(
        @{ $hParams->{'Attributes'} },
        {
            'SectionID'   => $Portal->alias,
            'SectionName' => $Portal->get( 'NameOrAlias', $hParams->{'LanguageID'} ),
            'Attributes'  => \@Attributes
        }
    );

    return;
}

1;
