#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::unit::CredentialsValidationRequest
# state        public
#----------------------------------------------------------------------------------------
# description  Unit test for CredentialsValidationRequest module.
#========================================================================================
package DE_EPAGES::DHLIntraship::t::unit::CredentialsValidationRequest;
use base DE_EPAGES::Test::API::UnitTest::Test;

use DE_EPAGES::Test::Mock::EpagesObject;
use Test::More;
use DateTime;
use DE_EPAGES::Core::API::Object::DateTimeFormatter;
use DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest;

use strict;


use constant DAYS_BEYOND_TODAY => 2;
use constant TEST_AUTHENTIFICATION_USER => 'MyTestUser';
use constant TEST_AUTHENTIFICATION_SIGNATURE => 'MyTestSignature';
use constant TEST_AUTHENTIFICATION_HASH => {
        'header' => {
            'user'      => TEST_AUTHENTIFICATION_USER,
            'signature' => TEST_AUTHENTIFICATION_SIGNATURE,
        }
};
use constant TEST_VERSION_HASH => {
        'Version' => {
            'majorRelease' => 3,
            'minorRelease' => 3,
        }
};
use constant TEST_FORMATTED_DATE => '2017-05-10';

## no critic (ProtectPrivateSubs)
## no critic (ProhibitPrivateSubroutineUse)

#========================================================================================
sub test {
    my $self = shift;

    $self->assembleAuthentification_test();
    $self->assembleVersion_test();
    $self->assembleManifestDate_test();
    $self->createManifestRequest_test();
    done_testing();
}

#========================================================================================
# Test methods
#========================================================================================
sub assembleAuthentification_test {
    my $self        = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hAuthentification =
            DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest->_assembleAuthentification(
                    TEST_AUTHENTIFICATION_USER, TEST_AUTHENTIFICATION_SIGNATURE
            );

    is_deeply($hAuthentification, TEST_AUTHENTIFICATION_HASH, "$SubTestName - Authentification hash assembled");
}

sub assembleVersion_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hVersion = DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest->_assembleVersion();

    is_deeply($hVersion, TEST_VERSION_HASH, "$SubTestName - Version hash assembled");
}

sub assembleManifestDate_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hManifestDate1 = DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest->_assembleManifestDate(
            { 'manifestDate' => TEST_FORMATTED_DATE }
    );

    is_deeply($hManifestDate1, { 'manifestDate' => TEST_FORMATTED_DATE },
            "$SubTestName - Manifest date hash (existing date) assembled");

    my $Formatter = DE_EPAGES::Core::API::Object::DateTimeFormatter->new();
    my $FormattedDate =
            $Formatter->format_datetime(DateTime->today()->add(days => DAYS_BEYOND_TODAY), 'date', '%Y-%m-%d');
    my $hManifestDate2 = DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest->_assembleManifestDate( { } );

    is_deeply($hManifestDate2, { 'manifestDate' => $FormattedDate },
            "$SubTestName - Manifest date hash (after tomorrow date) assembled");
}

sub createManifestRequest_test {
    my $self = shift;
    my $SubTestName = ( caller(0) )[3];

    my $hInputParameters = {
        'user'         => TEST_AUTHENTIFICATION_USER,
        'signature'    => TEST_AUTHENTIFICATION_SIGNATURE,
        'manifestDate' => TEST_FORMATTED_DATE
    };
    my $hManifestRequest =
            DE_EPAGES::DHLIntraship::API::SOAP::CredentialsValidationRequest->_createManifestRequest($hInputParameters);

    my $hExpectedAuth = TEST_AUTHENTIFICATION_HASH;
    my $hExpectedVersion = TEST_VERSION_HASH;

    is_deeply(
            $hManifestRequest,
            {
                %{ $hExpectedAuth },
                'part1' => {
                    %{ $hExpectedVersion },
                    'manifestDate' => TEST_FORMATTED_DATE
                }
            },
            "$SubTestName - Manifest request hash assembled"
    );
}


__PACKAGE__->runScript;