#========================================================================================
# package      DE_EPAGES::PayPalPPCP::Hooks::PaymentMethod
# state        public
# description  handles action for PaymentMethods
#========================================================================================
package DE_EPAGES::PayPalPPCP::Hooks::PaymentMethod;

use strict;

#========================================================================================
# function     OnInsertPaymentMethod
# state        public
#----------------------------------------------------------------------------------------
# syntax       OnInsertPaymentMethod( $hParams );
#----------------------------------------------------------------------------------------
# description  If OBJ_InsertPaymentMethod is triggered, deactivate a shop notification
#----------------------------------------------------------------------------------------
# input        $hParams | hook arguments, used key is
#               <ul>
#                   <li>Object
#               </ul> | ref.hash.*
#========================================================================================
sub OnInsertPaymentMethod {
    my ($hParams) = @_;

    # we deactivate the notification only if PPCP as payment was added
    my $PaymentMethod = $hParams->{'Object'};
    return unless $PaymentMethod->class->alias eq 'PaymentMethodPayPalPPCP';

    # deactivate shop notification if exists
    my $Shop = $PaymentMethod->getSite();
    if($Shop->folder('ShopNotifications')->existsChild('NewPayPalPPCPAvailable')){
        $Shop->folder('ShopNotifications')->child('NewPayPalPPCPAvailable')->deactivate();
    }

    return;
}

1;
