#==============================================================================
# package      DE_EPAGES::PayPalPPCP::UI::JSON
# base         DE_EPAGES::Presentation::UI::Object;
# state        private
#==============================================================================
package DE_EPAGES::PayPalPPCP::UI::JSON;
use base DE_EPAGES::Presentation::UI::JSON;

use strict;
use JSON;

#use DE_EPAGES::Core::API::PerlTools qw ( GetCurrentDateTime);
use DE_EPAGES::Core::API::Url qw ( BuildParameterString );
use DE_EPAGES::Object::API::Factory qw (LoadObjectByGUID);
use DE_EPAGES::Object::API::Table::Country qw( GetPKeyCountriesByCode2 );
use DE_EPAGES::Order::API::LineItemContainer qw ( SavePayment );
use DE_EPAGES::Order::API::PickupTokenExpiration;
use DE_EPAGES::Order::API::PickupTokenValidation;

use DE_EPAGES::PayPalPPCP::API::Constants qw ( $PPCP_ADDRESS_ALIAS );
use DE_EPAGES::PayPalPPCP::API::Object::PaymentMethodPayPalPPCP;


#========================================================================================
# function     JSONCreatePayPalPPCPOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       JSONApprovePayPalPPCPOrder( $Servlet );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#----------------------------------------------------------------------------------------
# description  Gets called from in UnitySF-PayPalPPCPScript.js
#               from the unity sf through a script tag
#               Approves PayPal order after successful paypal buy process
#========================================================================================
sub JSONApprovePayPalPPCPOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $BasketGUID = $Servlet->form->data()->{'BasketGUID'};
    my $FundingSource = $Servlet->form->data()->{'FundingSource'};
    my $Basket = LoadObjectByGUID($BasketGUID);
    my $Container = $Basket->container;

    if (!_CheckPickupBasketToken($Servlet, $Basket)) {
        return;
    };

    my $PaymentLine = $Container->get('Payment');
    my $PaymentMethod = $PaymentLine->get('PaymentMethod');
    my $hJSONResponse = $PaymentMethod->getPPCPOrder($PaymentLine);

    if ( ! $hJSONResponse ) {
        return;
    }

    my $hJSONPurchaseUnits = $hJSONResponse->{'purchase_units'};
    my $JSONBuyerAdress = @$hJSONPurchaseUnits[0]->{'shipping'}->{'address'};
    my $JSONBuyerName = @$hJSONPurchaseUnits[0]->{'shipping'}->{'name'};
    my $JSONBuyerEmail = $hJSONResponse->{'payer'}->{'email_address'};

    my $aCountryID = GetPKeyCountriesByCode2($JSONBuyerAdress->{'country_code'});
    my $CountryID = $aCountryID->[0];

    #-- If the address country is for one that ISN'T in the shop, do not allow this to continue
    my $aShopCountries = $Shop->get('ShopCountries');
    my $CountryValid = 0;
    foreach ( @$aShopCountries ) {
        if ( $CountryID == $_->{'CountryID'} ) {
            $CountryValid = 1;
            last;
        }
    }

    my $User = $Basket->get('User');
    my $UserAddresses = $User->folder('Addresses');
    my $IsAnonymous = $User->get('IsAnonymous') ? 1 : 0;

    my $Location;
    if ( $CountryValid ) {

        my ($Firstname, $Lastname) = _SplitFullname($JSONBuyerName->{'full_name'});

        my $AddressTypeToSet = 'BillingAddress';
        my $ExistingAddress = $Basket->get($AddressTypeToSet);
        if ( defined $Basket->get('ShippingAddress') ) {
            $AddressTypeToSet = 'ShippingAddress';
            $ExistingAddress = $Basket->get('ShippingAddress');
        }

        #my $Now = GetCurrentDateTime->epoch;
        my $PPCPAddressAlias = $PPCP_ADDRESS_ALIAS;
        $UserAddresses->child($PPCPAddressAlias)->delete if $UserAddresses->existsChild($PPCPAddressAlias);

        my $PayPalAddress = $UserAddresses->insertChild(
            'Address',
            {
                'Street'    => $JSONBuyerAdress->{'address_line_1'},
                'City'      => $JSONBuyerAdress->{'admin_area_2'},
                'State'     => $JSONBuyerAdress->{'admin_area_1'},
                'CountryID' => $CountryID,
                'Zipcode'   => $JSONBuyerAdress->{'postal_code'},
                'Alias'     => $PPCPAddressAlias,
                'FirstName' => substr($Firstname, 0, 50),
                'LastName'  => substr($Lastname, 0, 50),
                'EMail'     => $JSONBuyerEmail,
            }
        );

        #-- If we are anonymous, whatever the existing address, billing/shipping, delete it because we will set it to our PayPal address
        $ExistingAddress->delete if $ExistingAddress && $IsAnonymous;

        if ( ! $IsAnonymous ) {
            #-- We always want to use the billing address of the user for the basket to mimic the normal checkout process
            $Basket->set( { 'BillingAddress' => $User->get('BillingAddress') } );

            #-- We also need to update the shipping address of the user to our PayPal Address for the UnitySF templates
            $User->set( { 'ShippingAddress' => $PayPalAddress } );

            #-- Set the shipping address for logged in users
            $AddressTypeToSet = 'ShippingAddress';

            #-- At this point, there may be a billing/shipping AND our PayPal address under the User 'Addresses' folder
            #-- Unity only seems to allow for one billing and one shipping, so in the worst case there is one extra shipping address, our PayPal one
            #-- But we try and clean this up in hook API_OrderCreated
        }

        # Set Paypal addresses to the basket
        $Basket->set( {
            $AddressTypeToSet => $PayPalAddress,
        } );

        $Location = join '/',
            $Shop->get('WebServerScriptUrlSSL'),
            $Servlet->locale,
            '?'.BuildParameterString({
                ViewObjectID => $Basket->id,
                ViewAction => 'ViewMultiCheckoutConfirmation',
                ChangeObjectID => $Basket->id,
                PaymentMethod => $PaymentMethod->id,
                tac => '1',
                Fppcp => '1',   #-- Used to indicate that we are coming from the express button, see DE_EPAGES::PayPalPPCP::Hooks::Shop::OnLoadCheckoutCSS
            });

    } else {

        $Location = join '/',
            $Shop->get('WebServerScriptUrlSSL'),
            $Servlet->locale,
            '?'.BuildParameterString({
                ViewObjectID => $Basket->id,
                ViewAction => 'ViewMultiCheckoutAddress',
                ICPPCP => '1',   #-- Used to indicate that we are coming from the express button, and that the country is invalid
            });
    }

    #-- Clear the fund source and PUI, to reset inconsistent state if a customer goes back and forth, back and forth, baaaaaaaaaack, and foooooooorth
    $PaymentLine->set({
       'FundingSource' => $FundingSource ? $FundingSource : undef,  #-- Can only be PayPal, or 'Pay Later'
       'PayViaPUI' => 0,                                            #-- PUI can never be triggered from the cart page
    });

    my $hResult = {
        'success' => JSON::true,
        'redirect'  => $Location,
    };

    $self->writeContentOutput($hResult);
}

#========================================================================================
# function     JSONCreatePayPalPPCPOrder
# state        public
#----------------------------------------------------------------------------------------
# syntax       JSONCreatePayPalPPCPOrder( $Servlet );
#----------------------------------------------------------------------------------------
# input        $Servlet | servlet object | object
#----------------------------------------------------------------------------------------
# description  Gets called from in UnitySF-PayPalPPCPScript.js
#               from the unity sf trough a script tag
#               Creates PayPal order in paypal buy process
#========================================================================================
sub JSONCreatePayPalPPCPOrder {
    my $self = shift;
    my ($Servlet) = @_;

    my $Shop = $Servlet->object();
    my $BasketGUID = $Servlet->form->data()->{'BasketGUID'};
    my $Basket = LoadObjectByGUID($BasketGUID);

    my $Container = $Basket->container;
    my $NewPayPalPPCPPayment;
    foreach my $Payment (@{$Shop->get('PaymentMethods')}) {
        if($Payment->instanceOf('PaymentMethodPayPalPPCP') && $Payment->get('IsActivated')) {
            $NewPayPalPPCPPayment = $Payment;
        }
    }
    $Container->addLineItemPayment() unless $Container->get('Payment');
    SavePayment( $Container, $NewPayPalPPCPPayment );

    my $PaymentLine = $Container->get('Payment');

    #-- If for whatever reason we don't have a PPCP line item, we cannot proceed
    if ( ! defined $PaymentLine || ! $PaymentLine->instanceOf('LineItemPaymentPayPalPPCP') ) {
        return $self->writeContentOutput({
            'success' => JSON::false,
            'orderID'  => undef,
        });
    }

    #-- Set the 'IsExpressCheckout' flag to indicate that we have an express checkout
    $PaymentLine->set({
        'IsExpressCheckout' => 1,
    });

    my $hJsonResponse = $NewPayPalPPCPPayment->createPPCPOrder($PaymentLine);

    $PaymentLine->set({
        'OrderInfoJSON' => $hJsonResponse,
        'OrderPayload' => defined $hJsonResponse ? $NewPayPalPPCPPayment->getOrderPayload($PaymentLine) : undef,
        'TransID2'     => defined $hJsonResponse ? $hJsonResponse->{'id'} : undef,
    });

    my $hResult = {
        'success' => JSON::true,
        'orderID'  => $hJsonResponse->{'id'},
    };

    #-- Clear the fund source and PUI
    $PaymentLine->set({
       'FundingSource' => undef,
       'PayViaPUI' => 0,
    });

    $self->writeContentOutput($hResult);
}

sub _CheckPickupBasketToken {
    my ($Servlet, $Basket) = @_;

    return _CheckToken($Servlet, $Basket);
}

sub _CheckToken {
    my ($Servlet, $Basket) = @_;

    my $OrderCreationDate = $Basket->get('CreationDate');
    my $PickupTokenExpiration = _createPickupTokenExpiration();
    my $IsExpired = $PickupTokenExpiration->isExpired($OrderCreationDate);

    my $Form = $Servlet->form;
    my $PickupToken = $Form->data()->{'PickupToken'};

    my $PickupTokenValidation = _createPickupTokenValidation();
    my $UrlForValidation = $PickupTokenValidation->getUrlForPickupTokenValidation($Servlet);
    my $IsTokenValid = $PickupTokenValidation->checkPickupToken($PickupToken, $UrlForValidation);

    if ($IsTokenValid && !$IsExpired) {
        return 1;
    }

    return 0;
}

sub _createPickupTokenExpiration {
    return DE_EPAGES::Order::API::PickupTokenExpiration->new();
}

sub _createPickupTokenValidation {
    return DE_EPAGES::Order::API::PickupTokenValidation->new();
}

#========================================================================================
# function     SplitFullname
# state        private
#----------------------------------------------------------------------------------------
# syntax       ($Firstname, $Lastname) = SplitFullname( $hOptions );
#----------------------------------------------------------------------------------------
# description  Return the first- and lastname splitted from the fullname.
#----------------------------------------------------------------------------------------
# input        $Fullname | string
# return       ($Firstname, $Lastname) | array with First- and Lastname | array of strings
#========================================================================================
sub _SplitFullname ($) {
    my ( $Fullname ) = @_;

    my $Firstname;
    my $Lastname;

    if ($Fullname =~ /
                    (.*)    # any characters (lastname)
                    ,       # followed by a comma
                    [\s]*   # maybe followed by a whitespace character
                    (.*)   # followed by character (firstname and second name)
                    $
                    /x) {
        # Lastname,Firstname
        # Lastname, Firstname
        $Lastname = $1;
        $Firstname = $2;
    } elsif($Fullname =~ /(.*)\s(\S+)$/) {
        # Firstname Lastname
        $Firstname = $1;
        $Lastname = $2;
    } else {
        # Lastname
        $Firstname = '';
        $Lastname = $Fullname;
    }

    return ($Firstname, $Lastname);
}

1;
